require(Matrix)
require(tseries)

#self-paced function: hard scheme
eval_f_hard <- function(v, loss, lambda) {
	obj = v %*% loss - lambda*sum(v)
	return(obj)
}

#self-paced gradient: hard scheme
grad_f_hard <- function(v, loss, lambda) {
	grads = loss - lambda
	return(grads)
}

#self-paced function: linear scheme
eval_f_linear <- function(v, loss, lambda) {
	obj = v %*% loss + 1/2*lambda*sum(v^2) - lambda*sum(v)
	return(obj)
}

#self-paced gradient: linear scheme
grad_f_linear <- function(v, loss, lambda) {
	grads = loss + lambda * v - lambda
	return(grads)
}

#self-paced function: log scheme
eval_f_log <- function(v, loss, lambda) {
	zeta = 1-lambda
	obj = v %*% loss + zeta*sum(v) - sum(zeta^v)/log(zeta)
	return(obj)
}

#self-paced gradient: log scheme
grad_f_log <- function(v, loss, lambda) {
	zeta = 1-lambda
	grads = loss + zeta - zeta^v
	return(grads)
}

#self-paced function: mixture scheme
eval_f_mixture <- function(v, loss, lambda, lambda2 = 0.15) {
	#lambda here represents lambda1 in paper (lambda > lambda2 > 0)
	zeta = lambda*lambda2/(lambda-lambda2)
	obj = v %*% loss - zeta * sum(log(v+zeta/lambda))
	return(obj)
}

#self-paced gradient: mixture scheme
grad_f_mixture <- function(v, loss, lambda, lambda2 = 0.15) {
	#lambda here represents lambda1 in paper (lambda > lambda2 > 0)
	zeta = lambda*lambda2/(lambda-lambda2)
	grads = loss - zeta*lambda/(v*lambda+zeta)
	return(grads)
}

#closed-form solution: hard scheme
closedform_hard <- function(loss, lambda) {
	v = replicate(length(loss), 0)
	v[which(loss>=lambda)] = 0
	v[which(loss<lambda)] = 1
	return(v)
}

#closed-form solution: linear scheme
closedform_linear <- function(loss, lambda) {
	v = replicate(length(loss), 0)
	v = -1/lambda*loss+1
	v[which(loss>=lambda)]=0
	return(v)
}

#closed-form solution: log scheme
closedform_log <- function(loss, lambda) {
	zeta = 1-lambda
	v = replicate(length(loss), 0)
	v = 1/log(zeta)*log(loss+zeta)
	v[which(loss>=lambda)]=0
	return(v)
}

closedform_mixture <- function(loss, lambda, lambda2) {
	#lambda here represents lambda1 in paper (lambda > lambda2 > 0)
	zeta = (lambda*lambda2)/(lambda-lambda2)
	v = (lambda-loss)*zeta/(lambda*loss)
	v[which(loss>=lambda)] = 0
	v[which(loss<=lambda2)] = 1
	return(v)
}


#Obtain the curriculum (ordered by loss)
getcurriculum_hard <- function(solution, loss) {
	solution[which(solution>0.9)] = 1	#caliberate the solution to 0 and 1 (the solution may have some round errors)
	solution[which(solution<0.1)] = 0
	
	#rank the samples
	idx1 = which(solution==1)
	idx1 = idx1[sort(loss[idx1],index.return=TRUE)$ix]
	idx0 = which(solution==0)
	idx0 = idx0[sort(loss[idx0],index.return=TRUE)$ix]
	
	return(c(idx1,idx0))
}



